@@ -1,17 +1,14 @@
-# encoding: UTF-8
-
 require "minitest/autorun"
+require_relative "metametameta"
 
-if defined? Encoding then
-  e = Encoding.default_external
-  if e != Encoding::UTF_8 then
-    warn ""
-    warn ""
-    warn "NOTE: External encoding #{e} is not UTF-8. Tests WILL fail."
-    warn "      Run tests with `RUBYOPT=-Eutf-8 rake` to avoid errors."
-    warn ""
-    warn ""
-  end
+e = Encoding.default_external
+if e != Encoding::UTF_8 then
+  warn ""
+  warn ""
+  warn "NOTE: External encoding #{e} is not UTF-8. Tests WILL fail."
+  warn "      Run tests with `RUBYOPT=-Eutf-8 rake` to avoid errors."
+  warn ""
+  warn ""
 end
 
 SomeError = Class.new Exception
@@ -26,14 +23,11 @@ class TestMinitestAssertions < Minitest::Test
   # which is not threadsafe. Nearly every method in here is an
   # assertion test so it isn't worth splitting it out further.
 
-  RUBY18 = !defined? Encoding
-
   # not included in JRuby
   RE_LEVELS = /\(\d+ levels\) /
 
   class DummyTest
     include Minitest::Assertions
-    # include Minitest::Reportable # TODO: why do I really need this?
 
     attr_accessor :assertions, :failure
 
@@ -58,15 +52,6 @@ class TestMinitestAssertions < Minitest::Test
                  "expected #{@assertion_count} assertions to be fired during the test, not #{@tc.assertions}")
   end
 
-  def assert_deprecated name
-    dep = /DEPRECATED: #{name}. From #{__FILE__}:\d+(?::.*)?/
-    dep = "" if $-w.nil?
-
-    assert_output nil, dep do
-      yield
-    end
-  end
-
   def assert_triggered expected, klass = Minitest::Assertion
     e = assert_raises klass do
       yield
@@ -88,10 +73,6 @@ class TestMinitestAssertions < Minitest::Test
     end
   end
 
-  def clean s
-    s.gsub(/^ {6,10}/, "")
-  end
-
   def non_verbose
     orig_verbose = $VERBOSE
     $VERBOSE = false
@@ -144,36 +125,43 @@ class TestMinitestAssertions < Minitest::Test
   end
 
   def test_assert_equal_different_collection_array_hex_invisible
-    object1 = Object.new
-    object2 = Object.new
-    msg = "No visible difference in the Array#inspect output.
+    exp = Object.new
+    act = Object.new
+    msg = <<~EOM.chomp
+           No visible difference in the Array#inspect output.
            You should look at the implementation of #== on Array or its members.
-           [#<Object:0xXXXXXX>]".gsub(/^ +/, "")
+           [#<Object:0xXXXXXX>]
+        EOM
     assert_triggered msg do
-      @tc.assert_equal [object1], [object2]
+      @tc.assert_equal [exp], [act]
     end
   end
 
   def test_assert_equal_different_collection_hash_hex_invisible
-    h1, h2 = {}, {}
-    h1[1] = Object.new
-    h2[1] = Object.new
-    msg = "No visible difference in the Hash#inspect output.
+    exp, act = {}, {}
+    exp[1] = Object.new
+    act[1] = Object.new
+    act_obj = act[1]
+    # TODO: switch to endless when 2.7 is dropped
+    act_obj.define_singleton_method(:inspect) { "#<Object:0xXXXXXX>" }
+    msg = <<~EOM.chomp % [act]
+           No visible difference in the Hash#inspect output.
            You should look at the implementation of #== on Hash or its members.
-           {1=>#<Object:0xXXXXXX>}".gsub(/^ +/, "")
+           %p
+         EOM
 
     assert_triggered msg do
-      @tc.assert_equal h1, h2
+      @tc.assert_equal exp, act
     end
   end
 
   def test_assert_equal_different_diff_deactivated
     without_diff do
       assert_triggered util_msg("haha" * 10, "blah" * 10) do
-        o1 = "haha" * 10
-        o2 = "blah" * 10
+        exp = "haha" * 10
+        act = "blah" * 10
 
-        @tc.assert_equal o1, o2
+        @tc.assert_equal exp, act
       end
     end
   end
@@ -195,78 +183,84 @@ class TestMinitestAssertions < Minitest::Test
       def initialize s; @name = s; end
     end
 
-    o1 = c.new "a"
-    o2 = c.new "b"
-    msg = clean <<-EOS
+    exp = c.new "a"
+    act = c.new "b"
+    msg = <<~EOS
           --- expected
           +++ actual
           @@ -1 +1 @@
-          -#<#<Class:0xXXXXXX>:0xXXXXXX @name=\"a\">
-          +#<#<Class:0xXXXXXX>:0xXXXXXX @name=\"b\">
+          -#<#<Class:0xXXXXXX>:0xXXXXXX @name="a">
+          +#<#<Class:0xXXXXXX>:0xXXXXXX @name="b">
           EOS
 
     assert_triggered msg do
-      @tc.assert_equal o1, o2
+      @tc.assert_equal exp, act
     end
   end
 
   def test_assert_equal_different_hex_invisible
-    o1 = Object.new
-    o2 = Object.new
+    exp = Object.new
+    act = Object.new
 
-    msg = "No visible difference in the Object#inspect output.
+    msg = <<~EOM.chomp
+           No visible difference in the Object#inspect output.
            You should look at the implementation of #== on Object or its members.
-           #<Object:0xXXXXXX>".gsub(/^ +/, "")
+           #<Object:0xXXXXXX>
+          EOM
 
     assert_triggered msg do
-      @tc.assert_equal o1, o2
+      @tc.assert_equal exp, act
     end
   end
 
   def test_assert_equal_different_long
-    msg = "--- expected
+    msg = <<~EOM
+           --- expected
            +++ actual
            @@ -1 +1 @@
-           -\"hahahahahahahahahahahahahahahahahahahaha\"
-           +\"blahblahblahblahblahblahblahblahblahblah\"
-           ".gsub(/^ +/, "")
+           -"hahahahahahahahahahahahahahahahahahahaha"
+           +"blahblahblahblahblahblahblahblahblahblah"
+          EOM
 
     assert_triggered msg do
-      o1 = "haha" * 10
-      o2 = "blah" * 10
+      exp = "haha" * 10
+      act = "blah" * 10
 
-      @tc.assert_equal o1, o2
+      @tc.assert_equal exp, act
     end
   end
 
   def test_assert_equal_different_long_invisible
-    msg = "No visible difference in the String#inspect output.
+    msg = <<~EOM.chomp
+           No visible difference in the String#inspect output.
            You should look at the implementation of #== on String or its members.
-           \"blahblahblahblahblahblahblahblahblahblah\"".gsub(/^ +/, "")
+           "blahblahblahblahblahblahblahblahblahblah"
+          EOM
 
     assert_triggered msg do
-      o1 = "blah" * 10
-      o2 = "blah" * 10
-      def o1.== _
+      exp = "blah" * 10
+      act = "blah" * 10
+      def exp.== _
         false
       end
-      @tc.assert_equal o1, o2
+      @tc.assert_equal exp, act
     end
   end
 
   def test_assert_equal_different_long_msg
-    msg = "message.
+    msg = <<~EOM
+           message.
            --- expected
            +++ actual
            @@ -1 +1 @@
-           -\"hahahahahahahahahahahahahahahahahahahaha\"
-           +\"blahblahblahblahblahblahblahblahblahblah\"
-           ".gsub(/^ +/, "")
+           -"hahahahahahahahahahahahahahahahahahahaha"
+           +"blahblahblahblahblahblahblahblahblahblah"
+          EOM
 
     assert_triggered msg do
-      o1 = "haha" * 10
-      o2 = "blah" * 10
-      @tc.assert_equal o1, o2, "message"
+      exp = "haha" * 10
+      act = "blah" * 10
+      @tc.assert_equal exp, act, "message"
     end
   end
 
@@ -290,7 +284,7 @@ class TestMinitestAssertions < Minitest::Test
   end
 
   def test_assert_equal_does_not_allow_lhs_nil
-    if Minitest::VERSION =~ /^6/ then
+    if Minitest::VERSION >= "6" then
       warn "Time to strip the MT5 test"
 
       @assertion_count += 1
@@ -301,7 +295,7 @@ class TestMinitestAssertions < Minitest::Test
       err_re = /Use assert_nil if expecting nil from .*test_minitest_\w+.rb/
       err_re = "" if $-w.nil?
 
-      assert_output "", err_re do
+      assert_deprecation err_re do
         @tc.assert_equal nil, nil
       end
     end
@@ -314,29 +308,23 @@ class TestMinitestAssertions < Minitest::Test
   end
 
   def test_assert_equal_string_bug791
-    exp = <<-'EOF'.gsub(/^ {10}/, "") # note single quotes
-          --- expected
-          +++ actual
-          @@ -1,2 +1 @@
-          -"\\n
-          -"
-          +"\\\"
-        EOF
-
-    exp = "Expected: \"\\\\n\"\n  Actual: \"\\\\\""
+    exp = <<~EOM.chomp
+            Expected: "\\\\n"
+              Actual: "\\\\"
+          EOM
     assert_triggered exp do
       @tc.assert_equal "\\n", "\\"
     end
   end
 
   def test_assert_equal_string_both_escaped_unescaped_newlines
-    msg = <<-EOM.gsub(/^ {10}/, "")
+    msg = <<~'EOM' # NOTE: single quotes on heredoc
           --- expected
           +++ actual
           @@ -1,2 +1 @@
-          -\"A\\n
-          -B\"
-          +\"A\\n\\\\nB\"
+          -"A\n
+          -B"
+          +"A\n\\nB"
           EOM
 
     assert_triggered msg do
@@ -348,7 +336,7 @@ class TestMinitestAssertions < Minitest::Test
   end
 
   def test_assert_equal_string_encodings
-    msg = <<-EOM.gsub(/^ {10}/, "")
+    msg = <<~EOM
           --- expected
           +++ actual
           @@ -1,3 +1,3 @@
@@ -360,14 +348,14 @@ class TestMinitestAssertions < Minitest::Test
           EOM
 
     assert_triggered msg do
-      x = "bad-utf8-\xF1.txt"
-      y = x.dup.force_encoding "binary" # TODO: switch to .b when 1.9 dropped
-      @tc.assert_equal x, y
+      exp = "bad-utf8-\xF1.txt"
+      act = exp.dup.b
+      @tc.assert_equal exp, act
     end
-  end unless RUBY18
+  end
 
   def test_assert_equal_string_encodings_both_different
-    msg = <<-EOM.gsub(/^ {10}/, "")
+    msg = <<~EOM
           --- expected
           +++ actual
           @@ -1,3 +1,3 @@
@@ -379,14 +367,14 @@ class TestMinitestAssertions < Minitest::Test
           EOM
 
     assert_triggered msg do
-      x = "bad-utf8-\xF1.txt".force_encoding "ASCII"
-      y = x.dup.force_encoding "binary" # TODO: switch to .b when 1.9 dropped
-      @tc.assert_equal x, y
+      exp = "bad-utf8-\xF1.txt".dup.force_encoding Encoding::ASCII
+      act = exp.dup.b
+      @tc.assert_equal exp, act
     end
-  end unless RUBY18
+  end
 
   def test_assert_equal_unescape_newlines
-    msg = <<-'EOM'.gsub(/^ {10}/, "") # NOTE single quotes on heredoc
+    msg = <<~'EOM' # NOTE single quotes on heredoc
           --- expected
           +++ actual
           @@ -1,2 +1,2 @@
@@ -438,7 +426,7 @@ class TestMinitestAssertions < Minitest::Test
   end
 
   def test_assert_in_epsilon_triggered_negative_case
-    x = (RUBY18 and not maglev?) ? "0.1" : "0.100000xxx"
+    x = "0.100000xxx"
     y = "0.1"
     assert_triggered "Expected |-1.1 - -1| (#{x}) to be <= #{y}." do
       @tc.assert_in_epsilon(-1.1, -1, 0.1)
@@ -728,6 +716,7 @@ class TestMinitestAssertions < Minitest::Test
       end
     end
   end
+
   def test_assert_predicate
     @tc.assert_predicate "", :empty?
   end
@@ -757,17 +746,17 @@ class TestMinitestAssertions < Minitest::Test
       end
     end
 
-    expected = clean <<-EOM.chomp
+    expected = <<~EOM.chomp
       [StandardError] exception expected, not
       Class: <SomeError>
-      Message: <\"blah\">
+      Message: <"blah">
       ---Backtrace---
-      FILE:LINE:in \'block in test_assert_raises_default_triggered\'
+      FILE:LINE:in 'block in test_assert_raises_default_triggered'
       ---------------
     EOM
 
     actual = e.message.gsub(/^.+:\d+/, "FILE:LINE")
-    actual.gsub!(RE_LEVELS, "") unless jruby?
+    actual.gsub! RE_LEVELS, "" unless jruby?
     actual.gsub!(/[`']block in (?:TestMinitestAssertions#)?/, "'block in ")
 
     assert_equal expected, actual
@@ -837,17 +826,17 @@ class TestMinitestAssertions < Minitest::Test
       end
     end
 
-    expected = clean <<-EOM
+    expected = <<~EOM
       [SomeError] exception expected, not
       Class: <AnError>
-      Message: <\"some message\">
+      Message: <"some message">
       ---Backtrace---
-      FILE:LINE:in \'block in test_assert_raises_subclass_triggered\'
+      FILE:LINE:in 'block in test_assert_raises_subclass_triggered'
       ---------------
     EOM
 
     actual = e.message.gsub(/^.+:\d+/, "FILE:LINE")
-    actual.gsub!(RE_LEVELS, "") unless jruby?
+    actual.gsub! RE_LEVELS, "" unless jruby?
     actual.gsub!(/[`']block in (?:TestMinitestAssertions#)?/, "'block in ")
 
     assert_equal expected.chomp, actual
@@ -860,17 +849,17 @@ class TestMinitestAssertions < Minitest::Test
       end
     end
 
-    expected = clean <<-EOM.chomp
+    expected = <<~EOM.chomp
       [RuntimeError] exception expected, not
       Class: <SyntaxError>
-      Message: <\"icky\">
+      Message: <"icky">
       ---Backtrace---
-      FILE:LINE:in \'block in test_assert_raises_triggered_different\'
+      FILE:LINE:in 'block in test_assert_raises_triggered_different'
       ---------------
     EOM
 
     actual = e.message.gsub(/^.+:\d+/, "FILE:LINE")
-    actual.gsub!(RE_LEVELS, "") unless jruby?
+    actual.gsub! RE_LEVELS, "" unless jruby?
     actual.gsub!(/[`']block in (?:TestMinitestAssertions#)?/, "'block in ")
 
     assert_equal expected, actual
@@ -883,18 +872,18 @@ class TestMinitestAssertions < Minitest::Test
       end
     end
 
-    expected = clean <<-EOM
+    expected = <<~EOM
       XXX.
       [RuntimeError] exception expected, not
       Class: <SyntaxError>
-      Message: <\"icky\">
+      Message: <"icky">
       ---Backtrace---
-      FILE:LINE:in \'block in test_assert_raises_triggered_different_msg\'
+      FILE:LINE:in 'block in test_assert_raises_triggered_different_msg'
       ---------------
     EOM
 
     actual = e.message.gsub(/^.+:\d+/, "FILE:LINE")
-    actual.gsub!(RE_LEVELS, "") unless jruby?
+    actual.gsub! RE_LEVELS, "" unless jruby?
     actual.gsub!(/[`']block in (?:TestMinitestAssertions#)?/, "'block in ")
 
     assert_equal expected.chomp, actual
@@ -966,8 +955,8 @@ class TestMinitestAssertions < Minitest::Test
       @tc.assert_same 1, 2
     end
 
-    s1 = "blah"
-    s2 = "blah"
+    s1 = +"blah"
+    s2 = +"blah"
 
     assert_triggered 'Expected "blah" (oid=N) to be the same as "blah" (oid=N).' do
       @tc.assert_same s1, s2
@@ -975,16 +964,24 @@ class TestMinitestAssertions < Minitest::Test
   end
 
   def test_assert_send
-    assert_deprecated :assert_send do
+    @assertion_count = 0 if error_on_warn?
+    assert_deprecation(/DEPRECATED: assert_send/) do
       @tc.assert_send [1, :<, 2]
     end
   end
 
   def test_assert_send_bad
-    assert_deprecated :assert_send do
-      assert_triggered "Expected 1.>(*[2]) to return true." do
+    if error_on_warn? then
+      @assertion_count = 0
+      assert_deprecation(/DEPRECATED: assert_send/) do
         @tc.assert_send [1, :>, 2]
       end
+    else
+      assert_triggered "Expected 1.>(*[2]) to return true." do
+        assert_deprecation(/DEPRECATED: assert_send/) do
+          @tc.assert_send [1, :>, 2]
+        end
+      end
     end
   end
 
@@ -1155,8 +1152,8 @@ class TestMinitestAssertions < Minitest::Test
 
     non_verbose do
       out, err = capture_subprocess_io do
-        system("echo hi")
-        system("echo bye! 1>&2")
+        system "echo hi"
+        system "echo bye! 1>&2"
       end
 
       assert_equal "hi\n", out
@@ -1502,7 +1499,7 @@ class TestMinitestAssertions < Minitest::Test
     d0 = Time.now
     d1 = d0 + 86_400 # I am an idiot
 
-    assert_output "", /Stale skip_until \"not yet\" at .*?:\d+$/ do
+    assert_deprecation(/Stale skip_until \"not yet\" at .*?:\d+$/) do
       assert_skip_until d0, "not yet"
     end
 
@@ -1549,9 +1546,11 @@ class TestMinitestAssertionHelpers < Minitest::Test
   end
 
   def test_diff_equal
-    msg = "No visible difference in the String#inspect output.
+    msg = <<~EOM.chomp
+           No visible difference in the String#inspect output.
            You should look at the implementation of #== on String or its members.
-           \"blahblahblahblahblahblahblahblahblahblah\"".gsub(/^ +/, "")
+           "blahblahblahblahblahblahblahblahblahblah"
+          EOM
 
     o1 = "blah" * 10
     o2 = "blah" * 10
@@ -1563,7 +1562,7 @@ class TestMinitestAssertionHelpers < Minitest::Test
   end
 
   def test_diff_str_mixed
-    msg = <<-'EOM'.gsub(/^ {10}/, "") # NOTE single quotes on heredoc
+    msg = <<~'EOM' # NOTE single quotes on heredoc
           --- expected
           +++ actual
           @@ -1 +1 @@
@@ -1578,7 +1577,7 @@ class TestMinitestAssertionHelpers < Minitest::Test
   end
 
   def test_diff_str_multiline
-    msg = <<-'EOM'.gsub(/^ {10}/, "") # NOTE single quotes on heredoc
+    msg = <<~EOM
           --- expected
           +++ actual
           @@ -1,2 +1,2 @@
@@ -1594,7 +1593,7 @@ class TestMinitestAssertionHelpers < Minitest::Test
   end
 
   def test_diff_str_simple
-    msg = <<-'EOM'.gsub(/^ {10}/, "").chomp # NOTE single quotes on heredoc
+    msg = <<~EOM.chomp
           Expected: "A"
             Actual: "B"
           EOM
@@ -1646,14 +1645,14 @@ class TestMinitestAssertionHelpers < Minitest::Test
   end
 
   def test_mu_pp_for_diff_str_bad_encoding
-    str = "\666".force_encoding Encoding::UTF_8
+    str = "\666".dup.force_encoding Encoding::UTF_8
     exp = "# encoding: UTF-8\n#    valid: false\n\"\\xB6\""
 
     assert_mu_pp_for_diff exp, str, :raw
   end
 
   def test_mu_pp_for_diff_str_bad_encoding_both
-    str = "\666A\\n\nB".force_encoding Encoding::UTF_8
+    str = "\666A\\n\nB".dup.force_encoding Encoding::UTF_8
     exp = "# encoding: UTF-8\n#    valid: false\n\"\\xB6A\\\\n\\nB\""
 
     assert_mu_pp_for_diff exp, str, :raw
@@ -1700,7 +1699,7 @@ class TestMinitestAssertionHelpers < Minitest::Test
   end
 
   def test_mu_pp_str_bad_encoding
-    str = "\666".force_encoding Encoding::UTF_8
+    str = "\666".dup.force_encoding Encoding::UTF_8
     exp = "# encoding: UTF-8\n#    valid: false\n\"\\xB6\""
 
     assert_mu_pp exp, str, :raw
